import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
/// [Container]
class ImageWidget extends StatelessWidget {
  final String url;

  final String errorUrl;

  final String defUrl;

  final String package;

  final BoxFit fit;

  final VoidCallback onPressed;

  final VoidCallback onLongPress;

  final bool visible;

  final bool isClipOval;

  final double radius;

  final double width;

  final double height;

  final WidgetBuilder errorBuild;

  ImageWidget(
    this.url, {
    Key key,
    this.alignment,
    this.padding,
    this.color,
    this.decoration,
    this.foregroundDecoration,
    this.width,
    this.height,
    BoxConstraints constraints,
    this.margin,
    this.transform,
    this.clipBehavior = Clip.none,

    ///
    this.errorUrl,
    this.defUrl,
    this.onPressed,
    this.onLongPress,
    this.visible,
    this.package,
    this.isClipOval,
    this.radius,
    this.fit,
    this.errorBuild,
  })  : assert(margin == null || margin.isNonNegative),
        assert(padding == null || padding.isNonNegative),
        assert(decoration == null || decoration.debugAssertIsValid()),
        assert(constraints == null || constraints.debugAssertIsValid()),
        assert(clipBehavior != null),
        assert(decoration != null || clipBehavior == Clip.none),
        assert(
            color == null || decoration == null,
            'Cannot provide both a color and a decoration\n'
            'To provide both, use "decoration: BoxDecoration(color: color)".'),
        constraints = (width != null || height != null)
            ? constraints?.tighten(width: width, height: height) ??
                BoxConstraints.tightFor(width: width, height: height)
            : constraints,
        super(key: key);

  final AlignmentGeometry alignment;

  final EdgeInsetsGeometry padding;

  final Color color;

  final Decoration decoration;

  final Decoration foregroundDecoration;

  final BoxConstraints constraints;

  final EdgeInsetsGeometry margin;

  final Matrix4 transform;

  final Clip clipBehavior;

  EdgeInsetsGeometry get _paddingIncludingDecoration {
    if (decoration == null || decoration.padding == null) return padding;
    final EdgeInsetsGeometry decorationPadding = decoration.padding;
    if (padding == null) return decorationPadding;
    return padding.add(decorationPadding);
  }

  @override
  Widget build(BuildContext context) {
    Widget current = _buildImageView(context);

    if (constraints == null || !constraints.isTight) {
      current = LimitedBox(
        maxWidth: 0.0,
        maxHeight: 0.0,
        child: ConstrainedBox(constraints: const BoxConstraints.expand()),
      );
    }

    if (alignment != null)
      current = Align(alignment: alignment, child: current);

    final EdgeInsetsGeometry effectivePadding = _paddingIncludingDecoration;
    if (effectivePadding != null)
      current = Padding(padding: effectivePadding, child: current);

    if (color != null) current = ColoredBox(color: color, child: current);

    if (clipBehavior != Clip.none) {
      assert(decoration != null);
      current = ClipPath(
        clipper: _DecorationClipper(
            textDirection: Directionality.of(context), decoration: decoration),
        clipBehavior: clipBehavior,
        child: current,
      );
    }

    if (decoration != null)
      current = DecoratedBox(decoration: decoration, child: current);

    if (foregroundDecoration != null) {
      current = DecoratedBox(
        decoration: foregroundDecoration,
        position: DecorationPosition.foreground,
        child: current,
      );
    }

    if (constraints != null)
      current = ConstrainedBox(constraints: constraints, child: current);

    if (margin != null) current = Padding(padding: margin, child: current);

    if (transform != null)
      current = Transform(transform: transform, child: current);

    if (isClipOval == true) {
      current = ClipOval(
        child: current,
      );
    } else if (radius != null && radius > 0) {
      current = ClipRRect(
        borderRadius: BorderRadius.circular(radius),
        child: current,
      );
    }

    if (visible != null) current = Visibility(visible: visible, child: current);

    if (onPressed != null || onLongPress != null)
      current = GestureDetector(
          onTap: onPressed, onLongPress: onPressed, child: current);

    return current;
  }

  Widget _buildImageView(BuildContext context) {
    if (url == null || url.isEmpty || url == 'null') {
      return Container(
          width: width,
          height: height,
          color: color,
          child: _buildErrorWidget(context,
              msg: "url == null || url.isEmpty || url == 'null'"));
    }
    if (url.startsWith('https:') || url.startsWith('http:')) {
      return ImageWidgetComponent.instance.netWorkWidgetBuilder(this) ??
          Image.network(url, width: width, height: height, fit: fit,
              errorBuilder: (ctx, e, s) {
            return _buildErrorWidget(context, msg: "network error");
          });
    } else if (ImageWidgetComponent.instance.assetsNames
        .any((element) => url.startsWith(element))) {
      Widget result;
      assert(() {
        result = ImageWidgetComponent.instance.assetDebugBuilder?.call(this);
        return true;
      }());
      if (result != null) return result;

      var _package = package ?? ImageWidgetComponent.instance.defaultPackage;
      assert(_package != null && package.isNotEmpty, "未设置资源图片的bundle名称");
      if (package == "main") _package = null;
      if (url.endsWith(".svg")) {
        return ImageWidgetComponent.instance.svgWidgetBuilder(this) ??
            _buildErrorWidget(context, msg: "svgWidgetBuilder == null");
      }
      return Image.asset(url,
          fit: fit, package: _package, width: width, height: height);
    } else if (url.startsWith('file')) {
      return Image.file(File(url), fit: fit, width: width, height: height,
          errorBuilder: (c, o, s) {
        return _buildErrorWidget(context, msg: "File error path=$url");
      });
    } else if (url.startsWith('data:img/jpg;base64') ||
        url.startsWith('data:image/png;base64')) {
      try {
        return Image.memory(Base64Decoder().convert(url.split(',')[1] ?? ""),
            fit: fit, width: width, height: height);
      } catch (e) {
        return _buildErrorWidget(context, msg: "Base64Decoder error");
      }
    } else {
      return _buildErrorWidget(context, msg: "other error");
    }
  }

  Widget _buildErrorWidget(BuildContext context, {String msg}) {
    if (msg != null) debugPrint("ImageWidget msg:$msg \n ${this.toInfo()}");
    if (errorBuild != null) return errorBuild(context);
    return ImageWidgetComponent.instance.errorWidgetBuilder(this) ??
        Container(
            child: Icon(Icons.error_outline), width: width, height: height);
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(DiagnosticsProperty<AlignmentGeometry>(
        'alignment', alignment,
        showName: false, defaultValue: null));
    properties.add(DiagnosticsProperty<EdgeInsetsGeometry>('padding', padding,
        defaultValue: null));
    properties.add(DiagnosticsProperty<Clip>('clipBehavior', clipBehavior,
        defaultValue: Clip.none));
    if (color != null)
      properties.add(DiagnosticsProperty<Color>('bg', color));
    else
      properties.add(DiagnosticsProperty<Decoration>('bg', decoration,
          defaultValue: null));
    properties.add(DiagnosticsProperty<Decoration>('fg', foregroundDecoration,
        defaultValue: null));
    properties.add(DiagnosticsProperty<BoxConstraints>(
        'constraints', constraints,
        defaultValue: null));
    properties.add(DiagnosticsProperty<EdgeInsetsGeometry>('margin', margin,
        defaultValue: null));
    properties.add(ObjectFlagProperty<Matrix4>.has('transform', transform));
  }

  String toInfo() {
    return 'ImageWidget{url: $url, errorUrl: $errorUrl, defUrl: $defUrl, package: $package, fit: $fit, visible: $visible, isClipOval: $isClipOval, radius: $radius, width: $width, height: $height, alignment: $alignment, padding: $padding, color: $color, margin: $margin}';
  }
}

typedef ImageWidgetBuilder = Widget Function(ImageWidget);

class ImageWidgetComponent {
  static ImageWidgetComponent _instance;

  static ImageWidgetComponent get instance {
    return _instance ?? ImageWidgetComponent.init();
  }

  final List<String> assetsNames;
  final String defaultPackage;
  final ImageWidgetBuilder errorWidgetBuilder;
  final ImageWidgetBuilder svgWidgetBuilder;
  final ImageWidgetBuilder netWorkWidgetBuilder;
  final ImageWidgetBuilder assetDebugBuilder;

  ImageWidgetComponent._(
      this.errorWidgetBuilder,
      this.svgWidgetBuilder,
      this.netWorkWidgetBuilder,
      this.assetsNames,
      this.defaultPackage,
      this.assetDebugBuilder);

  factory ImageWidgetComponent.init(
      {List<String> assetsNames = const ['assets/', 'image/', 'images/'],
      ImageWidgetBuilder errorBuilder,
      ImageWidgetBuilder svgWidgetBuilder,
      ImageWidgetBuilder netWorkWidgetBuilder,
      ImageWidgetBuilder assetDebugBuilder,
      String defaultPackage}) {
    if (_instance == null) {
      _instance = ImageWidgetComponent._(errorBuilder, svgWidgetBuilder,
          netWorkWidgetBuilder, assetsNames, defaultPackage, assetDebugBuilder);
    }
    return _instance;
  }
}

/// A clipper that uses [Decoration.getClipPath] to clip.
class _DecorationClipper extends CustomClipper<Path> {
  _DecorationClipper({TextDirection textDirection, @required this.decoration})
      : assert(decoration != null),
        textDirection = textDirection ?? TextDirection.ltr;

  final TextDirection textDirection;
  final Decoration decoration;

  @override
  Path getClip(Size size) {
    return decoration.getClipPath(Offset.zero & size, textDirection);
  }

  @override
  bool shouldReclip(_DecorationClipper oldClipper) {
    return oldClipper.decoration != decoration ||
        oldClipper.textDirection != textDirection;
  }
}
